@@ -1,21 +1,20 @@
-cols_detrend_smoothed <- function(mat, mat_smoothed, purpose, seed, parallel) {
+cols_detrend_smoothed <- function(mat, mat_smoothed, purpose, parallel) {
   t(mat) %>%
     rows_detrend_smoothed(t(mat_smoothed), purpose = purpose,
-                          seed = seed, parallel = parallel) %>%
+                          parallel = parallel) %>%
     t()
 }
 
-cols_detrend_degree_specified <- function(mat, degree, purpose,
-                                          seed, parallel) {
+cols_detrend_degree_specified <- function(mat, degree, purpose, parallel) {
   smoothed <- poly_fit_cols(mat, degree, parallel = parallel)
   cols_detrend_smoothed(mat, smoothed, purpose = purpose,
-                        seed = seed, parallel = parallel)
+                        parallel = parallel)
 }
 
 cols_detrend_degree_specified_mean_b <- function(mat, degree, purpose,
-                                                 seed, parallel) {
+                                                 parallel) {
   cols_detrend_degree_specified(mat, degree, purpose = purpose,
-                                seed = seed, parallel = parallel) %>%
+                                parallel = parallel) %>%
     brightness_cols(parallel = parallel) %>%
     mean(na.rm = TRUE)
 }
@@ -43,29 +42,32 @@ cols_detrend_degree_specified_mean_b <- function(mat, degree, purpose,
 #' @examples
 #' \dontrun{
 #' ## These examples are not run on CRAN because they take too long.
-#' ## You should still try them for yourself.
+#' ## You can still try them for yourself.
 #' img <- ijtiff::read_tif(system.file('extdata', 'bleached.tif',
 #'                                     package = 'detrendr'))
-#' best_degree(img, seed = 0, parallel = 2)
+#' best_degree(img, parallel = 2)
 #' }
 #' @export
-best_degree <- function(img, seed = NULL, parallel = FALSE) {
+best_degree <- function(img, parallel = FALSE, purpose = c("FCS", "FFS")) {
   checkmate::assert_numeric(img, lower = 0)
   checkmate::assert_array(img, min.d = 3, max.d = 4)
   if (filesstrings::all_equal(img)) {
     stop("Your image is constant: all pixel values are equal to ",
          img[[1]], ". This type of image is not detrendable.")
   }
+  if (filesstrings::all_equal(purpose, c("FCS", "FFS")))
+    stop("You must choose *either* 'FCS' *or* 'FFS' for `purpose`.")
+  purpose %<>% filesstrings::match_arg(c("FCS", "FFS"), ignore_case = TRUE)
+  checkmate::assert(checkmate::check_flag(parallel),
+                    checkmate::check_count(parallel))
   d <- dim(img)
   if (length(d) == 3) {
-    if (is.null(seed)) seed <- rand_seed()
-    frame_length <- sum(!is.na(img[, , 1]))
+    frame_length <- sum(!anyNA_pillars(img))
     frame_means <- apply(img, 3, mean, na.rm = TRUE)
     sim_brightness <- NA
     for (i in 0:9) {
       if (is.na(sim_brightness)) {
-        sim_mat <- myrpois_frames_t(frame_means, frame_length,
-                                    seed + i, parallel)
+        sim_mat <- myrpois_frames_t(frame_means, frame_length, parallel)
         if (!filesstrings::all_equal(sim_mat)) {
           sim_brightness <- brightness_cols(sim_mat, parallel = parallel) %>%
             mean(na.rm = TRUE)
@@ -73,7 +75,8 @@ best_degree <- function(img, seed = NULL, parallel = FALSE) {
       }
     }
     msg <- paste("Your image is too close to zero. Can't detrend an image with",
-                 "so few nonzero values. \n* img has", length(img), "elements",
+                 "so few nonzero values. \n* `img` has",
+                 length(img), "elements",
                  "and just", sum(img > 0), "of them are greater than zero.")
     if (is.na(sim_brightness)) stop(msg)
     if (sim_brightness <= 1) return(NA)
@@ -81,8 +84,7 @@ best_degree <- function(img, seed = NULL, parallel = FALSE) {
     lower_degree_brightness <- sim_brightness
     upper_degree <- 1
     upper_degree_brightness <- cols_detrend_degree_specified_mean_b(
-      sim_mat, upper_degree, purpose = "ffs",
-      seed = seed, parallel = parallel)
+      sim_mat, upper_degree, purpose = purpose, parallel = parallel)
     if (is.na(upper_degree_brightness)) stop(msg)
     if (upper_degree_brightness < 1) {
       return(ifelse(1 - upper_degree_brightness < lower_degree_brightness - 1,
@@ -93,8 +95,7 @@ best_degree <- function(img, seed = NULL, parallel = FALSE) {
       lower_degree_brightness <- upper_degree_brightness
       upper_degree <- upper_degree + 1
       upper_degree_brightness <- cols_detrend_degree_specified_mean_b(
-        sim_mat, upper_degree, purpose = "ffs",
-        seed = seed, parallel = parallel)
+        sim_mat, upper_degree, purpose = purpose, parallel = parallel)
       if (is.na(upper_degree_brightness)) stop(msg)
     }
     out <- ifelse(1 - upper_degree_brightness < lower_degree_brightness - 1,
@@ -108,7 +109,7 @@ best_degree <- function(img, seed = NULL, parallel = FALSE) {
     as.integer(out)
   } else {
     purrr::map_int(seq_len(d[3]),
-                   ~ best_degree(img[, , ., ],
-                                 seed = seed, parallel = parallel))
+                   ~ best_degree(img[, , ., ], purpose = purpose,
+                                 parallel = parallel))
   }
 }
